#include <Servo.h> 

#define LED_PIN 13 
#define LEFT_EYE_LED_PIN 7
#define RIGHT_EYE_LED_PIN 6

#define LED_PULSE_WIDTH 500

#define SERVO_UPDATE_SPEED 9

#define SERVO_SLOWEST_UPDATE_SPEED 9

#define SERIAL_SPEED 115200
#define SERIAL_STATUS_SYMBOL '0'
#define SERIAL_GO_UP_OSCILATE_SYMBOL '1'
#define SERIAL_GO_UP_SYMBOL '2'
#define SERIAL_GO_DOWN_OSCILATE_SYMBOL '3'
#define SERIAL_GO_DOWN_SYMBOL '4'
#define SERIAL_SLOW_UP_SYMBOL '8'
#define SERIAL_SLOW_DOWN_SYMBOL '9'
#define SERIAL_TURN_ON_EYES_SYMBOL 'e'
#define SERIAL_TURN_OFF_EYES_SYMBOL 's'
#define SERIAL_OK_SYMBOL "OK"
#define SERIAL_READY_SYMBOL "READY"

#define SHORT_DELAY 15

#define SERVO_PIN 9
#define SERVO_MIN_POSITION 150
#define SERVO_MAX_POSITION 90

Servo servo;

int incomingByte = 0;

int servoTargetPosition = SERVO_MAX_POSITION;
unsigned long timeServoChecked = 0;

boolean isLedOn = false;
unsigned long timeLedOn = 0;

//Flash Led, start now and,  checkLed will turn off when necesary
void flashLed() {
  digitalWrite(LED_PIN, HIGH);
  timeLedOn = millis();
  isLedOn = true;
}

//Turn off led if time out
void checkLed() {
  if (isLedOn) {
    if (millis() > timeLedOn + LED_PULSE_WIDTH) {
      digitalWrite(LED_PIN, LOW);
      isLedOn = false;
    }
  }
}

void updateServoPosition() {
  if (millis() > timeServoChecked + SERVO_UPDATE_SPEED) {
    int currentServoPosition = servo.read();
    int newServoPosition = 0;
  
    if (servoTargetPosition == currentServoPosition) {
        newServoPosition = currentServoPosition;
        
    }else if (servoTargetPosition < currentServoPosition) {
      newServoPosition = currentServoPosition - 1;
      
    }else {
      newServoPosition = currentServoPosition + 1;
      
    }
  servo.write(newServoPosition);
  timeServoChecked = millis();
  }
}

void moveServoToPosition(int position) {
  servoTargetPosition = position;
}

//Move servo to position with a short oscilation
void oscilateServo(int destinationPosition) {
  int oscilatePosition = 0;
  int endPosition = destinationPosition;
     
  if(destinationPosition >= 10) {
    oscilatePosition = destinationPosition - 10;
  }else {
    oscilatePosition = destinationPosition + 10;
  }
  servo.write(oscilatePosition);
  delay(SHORT_DELAY);
  servo.write(endPosition);
  delay(SHORT_DELAY);
}
 
void setup() { 
  //Initialize servo
  servo.attach(SERVO_PIN);
  oscilateServo(SERVO_MAX_POSITION);
  
  //Initialize serial
  Serial.begin(SERIAL_SPEED); 
  
  //Initialize pin
  pinMode(LED_PIN, OUTPUT);
  flashLed();

  //Initialize eye leds pin
  pinMode(LEFT_EYE_LED_PIN, OUTPUT);
  digitalWrite(LEFT_EYE_LED_PIN, HIGH);
  pinMode(RIGHT_EYE_LED_PIN, OUTPUT);
  digitalWrite(RIGHT_EYE_LED_PIN, HIGH);
  
  //Send ready symbol
  Serial.println(SERIAL_READY_SYMBOL); 
  delay(SHORT_DELAY);

} 

 
void loop() {
  
  checkLed();
  
  updateServoPosition();
  
  if (Serial.available() > 0) {
    incomingByte = Serial.read();
    Serial.flush();
    
    //Check incoming symbol    
    switch (char(incomingByte)) 
    {
      case SERIAL_GO_UP_SYMBOL:
        flashLed();
        servo.write(SERVO_MAX_POSITION);
        Serial.print(SERIAL_GO_UP_SYMBOL); 
        Serial.println(SERIAL_OK_SYMBOL); 
        delay(SHORT_DELAY);
        break;
        
      case SERIAL_GO_UP_OSCILATE_SYMBOL:
        flashLed();
        oscilateServo(SERVO_MAX_POSITION);
        Serial.print(SERIAL_GO_UP_OSCILATE_SYMBOL); 
        Serial.println(SERIAL_OK_SYMBOL); 
        delay(SHORT_DELAY);
        break;
        
      case SERIAL_GO_DOWN_SYMBOL:
        flashLed();
        servo.write(SERVO_MIN_POSITION);
        Serial.print(SERIAL_GO_DOWN_SYMBOL); 
        Serial.println(SERIAL_OK_SYMBOL); 
        delay(SHORT_DELAY);
        break;
        
      case SERIAL_GO_DOWN_OSCILATE_SYMBOL:
        flashLed();
        oscilateServo(SERVO_MIN_POSITION);
        Serial.print(SERIAL_GO_DOWN_OSCILATE_SYMBOL); 
        Serial.println(SERIAL_OK_SYMBOL); 
        delay(SHORT_DELAY);
        break;
        
      case SERIAL_TURN_ON_EYES_SYMBOL:
        digitalWrite(LEFT_EYE_LED_PIN, HIGH);
        digitalWrite(RIGHT_EYE_LED_PIN, HIGH);
        Serial.println(SERIAL_OK_SYMBOL); 
        delay(SHORT_DELAY);
        break;
        
      case SERIAL_TURN_OFF_EYES_SYMBOL:
        digitalWrite(LEFT_EYE_LED_PIN, LOW);
        digitalWrite(RIGHT_EYE_LED_PIN, LOW);
        Serial.println(SERIAL_OK_SYMBOL); 
        delay(SHORT_DELAY);
        break;
        
      case SERIAL_SLOW_UP_SYMBOL:
        flashLed();
        moveServoToPosition(SERVO_MAX_POSITION);
        Serial.print(SERIAL_SLOW_UP_SYMBOL); 
        Serial.println(SERIAL_OK_SYMBOL); 
        delay(SHORT_DELAY);
        break;
        
      case SERIAL_SLOW_DOWN_SYMBOL:
        flashLed();
        moveServoToPosition(SERVO_MIN_POSITION);
        Serial.print(SERIAL_SLOW_DOWN_SYMBOL); 
        Serial.println(SERIAL_OK_SYMBOL); 
        delay(SHORT_DELAY);
        break;
        
      case SERIAL_STATUS_SYMBOL:
      default:
        Serial.print(servo.read(), DEC); 
        Serial.println(SERIAL_READY_SYMBOL); 
        delay(SHORT_DELAY);
        break;
        
    }
      
  }
  
}
